# Row Polymorphic Programming

```idris hide
module LessMacrosMoreTypes.RowPolymorphism

import DependentNuggets.FreshLists
```

Sometimes, especially when dealing with business logic, we have to deal with data that comes in from the real world, or elsewhere, naturally very messily typed, leading to nasty type signatures and messes of macros or code generation to build data structures from a schema. Row polymorphism can help us by abstracting over the fields contained in a record type in the type signature, letting us define records based on data, concatenate records together, and define functions that are generic across any record containing the required fields, all without macros.

This article is inspired by my work on implementing the _Brown Benchmark for Table Types_(B2T2)[^1] for my _Structures_[^2] library, an update for which will be released soon, providing a much more comprehensive collection of types for row-polymorphic programming.


## What is Row Polymorphism

At time of writing, the [Wikipedia page](https://en.wikipedia.org/wiki/Row_polymorphism) for row polymorphism starts with 
> In programming language type theory, row polymorphism is a kind of polymorphism that allows one to write programs that are polymorphic on row types such as record types and polymorphic variants.
and then leads into some mathematical notation heavy text that doesn't really elucidate much, and what the hell does "polymorphic on row types" even mean?

Consider a basic `record` in Idris as an example:
```idris
record ImACat where
  constructor MkCat
  name : String
  age : Nat
  meows_a_lot : Bool
```

This certainly seems like it might fit the bill, it says `record` right there. But what are these "row types"?

### Row Types

Row types often are called records in other contexts, and while "record" may be a somewhat more intuitive name for someone approaching this from the perspective of many common programming languages, I will continue to largely use the name "row type" as it will make more sense in the tabular data context I will be framing the rest of this post in terms of.

Consider some tabular data, say a table in a database, or a spread sheet. Tabular data is composed of a 2d grid of rows and columns, typically with each column having a name and potentially a type constraint on the values in that column. Consider this example of some tabular data: 

<style type="text/css">
#tbl-cats tbody tr :nth-child(2) {
    text-align: right;
}
</style>
<div class="md-table" id="tbl-cats">
<span class="md-table-title">Cats</span>

| name → String | age → Nat | meows_a_lot →  Bool |
|---------------|-----------|---------------------|
| Cricket       | 11        | False               |
| Cookie-Dough  | 4         | True                |
| Mango         | 0         | False               |
| Creampuff     | 5         | True                |

</div>

A row type is parameterized by a list of `(label, Type)` pairs, represented by the header of the table. We can see that the labels and types in our *Cats* table are identical to the fields of our `ImACat` record from earlier. Under this view, each row in the table can be thought of as an instance of the `ImACat` record. 

That parameterization on a schema, however, makes row types a bit more flexible, as well as a bit more complicated. We can perform the usual operations, like accessing a column in a row by name, but we can also perform new operations that enable a sort of limited, type-safe reflection over row types, such as selecting a specific set of columns from a row to produce a new row type, or concatenating the schemas of two rows to create a new row type expressing the result of joining the two rows.

### The Mysterious Row Polymorphism

Imagine you are helping your friend move and they need to figure out what furniture will fit in what rooms. The current property owner of their new place has graciously provided a table containing the length and width of each room, and their furniture suppler has provided a table detailing the lengths, widths, and heights of the bounding cuboids for each piece of furniture. Let's say the tables look a bit like this:

<style type="text/css">
#tbl-rooms tbody tr :nth-child(2),
#tbl-rooms tbody tr :nth-child(3) {
    text-align: right;
}
</style>
<div class="md-table" id="tbl-rooms">
<span class="md-table-title">Rooms</span>

| room → String | length → Double | width → Double |
|---------------|-----------------|----------------|
| Bedroom       | 3.00            | 6.00           |
| Kitchen       | 2.50            | 2.00           |
| Bathroom      | 1.50            | 1.00           |

</div>

<style type="text/css">
#tbl-furniture tbody tr :nth-child(2), 
#tbl-furniture tbody tr :nth-child(3), 
#tbl-furniture tbody tr :nth-child(4) {
    text-align: right;
}
</style>
<div class="md-table" id="tbl-furniture">
<span class="md-table-title">Furniture</span>

| item → String | length → Double | width → Double | height → Double |
|---------------|-----------------|----------------|-----------------|
| Dresser       | 1.00            | 0.25           | 0.80            |
| Chair         | 0.50            | 0.50           | 1.00            |
| Nightstand    | 0.75            | 0.50           | 0.50            |

</div>

One thing that's going to be critical to determine is how much floor space is left in a room after the furniture's been placed in it. After all, a room isn't of much use if there's not even enough free floor space to stand in it. You notice that both of these tables contain the information needed to calculate the available area and the foot prints of the furniture, the `length` and `width` columns.

Its tempting to define a `Room` data type and a `Furniture` data type containing each table's respective columns as fields, and describe the surface area/footprint via an `Area` interface. However, this approach has several drawbacks. It requires implementing the interface manually for each data type[^3], can hide certain categories of error like inadvertently using the wrong field to calculate the area, and is potentially quite fragile if the structure of the tables changes on us or we need to add more tables to our system.

We can do better. With our parameterized row types we can write an `area` function that is defined to work on any row type that has a `length` and a `width` column of type `Double`. This is the essence of row polymorphism, writing code that's generic not over what interfaces a type implements, but instead is generic over what fields the type contains.

## Building our own Runtime Reflection

### Building the Schema

### Computing Our Table Types

## Making and Using tables

## Things this blog post's implementation does wrong

[^1]: <https://github.com/brownplt/B2T2>

[^2]: <https://git.sr.ht/~thatonelutenist/Structures>

[^3]: At best, we might be able to get away with something like a derive macro, but that's still something we have to invoke manually
